#!/usr/bin/python3
#-----------------------------------------------------------------------
# File    : run.py
# Contents: best response regression experiments
# Author  : Christian Borgelt
# History : 2020.03.22 file created
#           2020.03.26 adapted to extended xdist parameter
#           2020.03.31 adapted to objective function parameter
#-----------------------------------------------------------------------
from sys        import argv, stdout, stderr
from subprocess import call, check_output, STDOUT
from os         import makedirs, rename, remove, devnull
from time       import strftime
from glob       import glob
from id2par     import get_args, run

#-----------------------------------------------------------------------

def brreg (mode='resi', ab=(0.0,1.0), size=100,
           xdist=('n',2.0), error=('n',1.0),
           objfn='FR', eps=0.1, beta=1.0, gamma=64.0,
           trials=100, eta=0.001, thresh=1e-6, steps=100,
           facts=(0.5,1.2), change=(1e-6,1.0), runs=100, rseed=0):
    fname = 'data/n%d_x%s_e%s_o%s_m%s_i0%03d_b%g_g%g.txt' \
            % (size,xdist[0],error[0],
               objfn,mode,int(eps*1000),beta,gamma)
    cmd   = ['./brreg', '-ddata/', '%s'    % mode,
             '-a%g:%g' % ab,     '-n%d'    % size,
             '-x%s:%g' % xdist,  '-e%s:%g' % error,
             '-o%s'    % objfn,  '-i%g'    % eps,
             '-b%g'    % beta,   '-g%g'    % gamma,   
             '-t%d'    % trials, '-l%g'    % eta,
             '-T%g'    % thresh, '-s%d'    % steps,
             '-f%g:%g' % facts,  '-z%g:%g' % change,
             '-r%d'    % runs,   '-S%d'    % rseed]
    print(strftime('%Y.%m.%d %H:%M:%S brreg ') +fname +' start.')
    try: makedirs('data')       # ensure result directory exists
    except OSError: pass        # and execute created command
    with open(fname, 'w') as out, open(devnull, 'w') as null:
        call(cmd, stdout=out, stderr=null)
    print(strftime('%Y.%m.%d %H:%M:%S brreg ') +fname +' done.')
    
#-----------------------------------------------------------------------

def collect ():
    files = glob('data/*.txt')
    res   = []
    for f in files:
        r = [s[1:] for s in f.split('.')[0].split('_')]
        with open(f, 'r') as inp:
            for line in inp: pass
        r += line.rstrip('\r\n').split()[1:]
        r[0] = r[0][5:]
        r[5] = '0.'+r[5][1:]
        res.append(r)
    with open('brreg.txt', 'w') as out:
        out.write('points\txdist\terror\tobjfn\tmethod'
                  '\tepsilon\tbeta\tgamma\twtrn\twtst\twsse\n')
        for r in sorted(res):
            out.write('\t'.join(r))
            out.write('\n')

#-----------------------------------------------------------------------

if __name__ == '__main__':
    grids = [ [ ( { 'objfn': '--', 'mode': 'rbst', 'trials':  20 },
                  { 'objfn': 'FH', 'mode': 'boot', 'trials': 100 },
                  { 'objfn': 'FL', 'mode': 'boot', 'trials': 100 },
                  { 'objfn': 'FR', 'mode': 'boot', 'trials': 100 },
                  { 'objfn': 'FL', 'mode': 'grad', 'trials':  20 },
                  { 'objfn': 'FL', 'mode': 'resi', 'trials':  20 },
                  { 'objfn': 'FL', 'mode': 'newt', 'trials':  20 },
                  { 'objfn': 'FR', 'mode': 'grad', 'trials':  20 },
                  { 'objfn': 'FR', 'mode': 'resi', 'trials':  20 },
                  { 'objfn': 'FR', 'mode': 'newt', 'trials':  20 } ),
                ( 'ab',     [ (0.0,1.0) ] ),
                ( 'size',   [ 20, 50, 100 ] ),
                ( 'xdist',  [ ('n', 2.0), ('u', 2.0) ] ),
                ( 'error',  [ ('n', 1.0), ('2', 1.0), ('3', 1.0),
                              ('u', 1.0), ('l', 1.0), ('c', 1.0) ] ),
                ( 'eps',    [ 0.1, 0.01, 0.001 ] ),
                ( 'beta',   [ 1.0, 3.0, 6.0, 10.0 ] ),
                ( 'gamma',  [ 32    ] ),
                ( 'eta',    [ 0.001 ] ),
                ( 'thresh', [ 1e-6  ] ),
                ( 'steps',  [ 200   ] ),
                ( 'facts',  [ ( 0.5,1.2) ] ),
                ( 'change', [ (1e-6,1.0) ] ),
                ( 'runs',   [ 10000 ] ),
                ( 'rseed',  [ 1     ] )
            ] ]

    if len(argv) < 2 or argv[1] != 'collect':
        run(brreg, grids, int(argv[1])-1 if len(argv) > 1 else -1)
    else:                       # run experiments
        collect()               # or collect results
